import logging
log = logging.getLogger(__name__)

from collections import namedtuple
from numpy import append, array, interp, linspace

"""
A waveform generator.
"""


Waveform = namedtuple('Waveform', 'data, markers')


class Generator(object):
	"""
	A generator for arbitrary waveforms.
	"""

	# Generation should fail if the number of points exceeds this value.
	max_length = 10000000 # 1e7 (0.01 s @ 1 GHz)

	length = 0

	def __init__(self, frequency, dry_run=False):
		# The sampling frequency.
		self.frequency = frequency

		# If True, do not generate a waveform. Useful for verifying the generating code.
		self.dry_run = dry_run

		# The resulting wave, with each data point on the interval [-1.0, 1.0].
		self._wave = array([])

		# The resulting marker channels, with each channel being a sparse list represented as a dictionary.
		self._markers = {}

	@property
	def waveform(self):
		"""
		The waveform and marker data for the generated waveform.
		"""

		try:
			last_marker_point = max(pos for data in self._markers.values() for pos in data.keys())
		except ValueError:
			last_marker_point = -1

		extra_points = last_marker_point + 1 - len(self._wave)
		if extra_points > 0:
			resulting_wave = append(self._wave, [0.0] * extra_points)
		else:
			resulting_wave = self._wave

		marker_data = dict((num, self._get_marker(num, len(resulting_wave))) for num in self._markers)

		return Waveform(resulting_wave, marker_data)

	def check_length(self, additional):
		resulting_length = self.length + additional

		if resulting_length > self.max_length:
			raise ValueError('Waveform is too long; stopping at {0:n} points'.format(resulting_length))

	def append(self, values):
		self.length += len(values)

		if not self.dry_run:
			self._wave = append(self._wave, values)

	def _get_marker(self, num, length):
		"""
		Get the marker values for all data points in the waveform.
		"""

		result = []

		def last_value():
			try:
				return result[-1]
			except IndexError:
				return False

		for idx, value in sorted(self._markers[num].items()):
			if idx >= len(result):
				result.extend([last_value()] * (idx - len(result) + 1))

			result[idx] = value

		if len(result) < length:
			result.extend([last_value()] * (length - len(result)))

		return result

	def _parse_time(self, value):
		"""
		Convert a time value to a number of samples based on the frequency.
		"""

		log.debug('Parsing time "{0!r}" with frequency {1!r}'.format(value, self.frequency))
		result = int(value.value * self.frequency.value)
		log.debug('Parsed time "{0!r}" as {1} samples'.format(value, result))

		return result

	def _scale_waveform(self, data, amplitude=None, duration=None):
		"""
		Shorten or elongate a waveform in both axes.

		Due to the discrete nature of these waveforms, interpolation is used when changing duration.
		"""

		if not data:
			return data

		new_data = data[:]

		# Change amplitude.
		if amplitude is not None:
			new_data = [amplitude * x for x in new_data]

		# Change duration.
		if duration is not None:
			duration = self._parse_time(duration)
			actual_duration = len(new_data)

			points = linspace(0, 1, duration)
			actual_points = linspace(0, 1, actual_duration)

			new_data = interp(points, actual_points, new_data)
			new_data = [round(x, 5) for x in new_data]

		return new_data

	def set_next(self, value):
		"""
		Set the next point to have the given amplitude.
		"""

		self.check_length(1)
		self.append([value])

	def delay(self, value, less_points=1):
		"""
		Extend the last value of the waveform to last the length of the delay.
		"""

		delay_length = self._parse_time(value) - less_points

		self.check_length(delay_length)

		try:
			last_value = self._wave[-1]
		except IndexError:
			last_value = 0.0

		self.append([last_value] * delay_length)

	def square(self, amplitude, length):
		"""
		Generate a square pulse.
		"""

		try:
			return_to = self._wave[-1]
		except IndexError:
			return_to = 0.0

		self.set_next(amplitude)
		self.delay(length, less_points=1)
		self.set_next(return_to)

	def pulse(self, values, amplitude, duration):
		"""
		Literal amplitude values.
		"""

		data = self._scale_waveform(values, amplitude, duration)

		self.check_length(len(data))
		self.append(data)

	def marker(self, num, value):
		"""
		Set the value of a marker starting from the current position.
		"""

		if self.dry_run:
			return

		if num not in self._markers:
			self._markers[num] = {}

		self._markers[num][len(self._wave)] = value



from ...mock.mock_abstract_device import MockAbstractDevice
from ..ips120_10 import IPS120_10

"""
Mock Sample IPS120_10
"""


class MockIPS120_10(MockAbstractDevice, IPS120_10):
	"""
	Mock interface for the Sample IPS120_10.
	"""

	def __init__(self, *args, **kwargs):
		self.mocking = IPS120_10

		self.heater_delay = 0

		MockAbstractDevice.__init__(self, *args, **kwargs)

	def _reset(self):
		self.mock_state['heater_on'] = False
		self.mock_state['activity'] = 4
		self.mock_state['mode'] = 0

		self.mock_state['output_field'] = 0.0
		self.mock_state['set_point'] = 0.0
		self.mock_state['sweep_rate'] = 1.0
		self.mock_state['persistent_field'] = 0.0

	def _split_message(self, message):
		if message[0] == '$':
			query = False
			message = message[1:]
		else:
			query = True

		cmd = message[0]
		args = message[1:]

		return (cmd, args, query)

	def write(self, message, result=None, done=False):
		if not done:
			cmd, args, query = self._split_message(message)

			if cmd in ['C', 'M', 'Q']:
				done = True
			elif cmd == 'A' and not query:
				self.mock_state['activity'] = int(args)
				if self.mock_state['activity'] == 0:
					done = True
				elif self.mock_state['activity'] == 1:
					self.mock_state['output_field'] = self.mock_state['set_point']
					done = True
			elif cmd == 'H' and not query:
				self.mock_state['heater_on'] = (args == '1')
				if not self.mock_state['heater_on']:
					self.mock_state['persistent_field'] = self.mock_state['output_field']
				done = True
			elif cmd == 'J' and not query:
				self.mock_state['set_point'] = float(args)
				done = True
			elif cmd == 'R':
				if query:
					result = 'R'
					if args == '7':
						result += str(self.mock_state['output_field'])
					if args == '8':
						result += str(self.mock_state['set_point'])
					if args == '9':
						result += str(self.mock_state['sweep_rate'])
					elif args == '18':
						result += str(self.mock_state['persistent_field'])
				done = True
			elif cmd == 'T' and not query:
				self.mock_state['sweep_rate'] = float(args)
				done = True
			elif cmd == 'X':
				if query:
					# 9 is used for the don't-cares
					result = 'X00A{0}C9H{1}M9{2}'.format(self.mock_state['activity'],
							str(int(self.mock_state['heater_on'])), self.mock_state['mode'])
				done = True


		MockAbstractDevice.write(self, message, result, done)


name = 'IPS120-10'
implementation = MockIPS120_10



import logging
log = logging.getLogger(__name__)


name = 'Oxford Instruments'

from . import ips120_10
models = [ips120_10]
log.debug('Found models for "{0}": {1}'.format(name, ''.join(str(x) for x in models)))

from .mock import mock_ips120_10
mock_models = [mock_ips120_10]
log.debug('Found mock models for "{0}": {1}'.format(name, ''.join(str(x) for x in mock_models)))



from numpy import array
import wx

from spacq.interface.list_columns import ListParser
from spacq.tool.box import triples_to_mesh

from ....tool.box import MessageDialog
from ..surface import SurfacePlot
from .common.plot_setup import PlotSetupDialog


class SurfacePlotPanel(wx.Panel):
	def __init__(self, parent, surface_data, x_bounds, y_bounds, x_label, y_label, z_label,
			style, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Plot.
		self.plot = SurfacePlot(self, style=style)
		panel_box.Add(self.plot.control, proportion=1, flag=wx.EXPAND)

		self.SetSizer(panel_box)

		self.plot.x_label, self.plot.y_label, self.plot.z_label = x_label, y_label, z_label
		self.plot.surface_data = (surface_data, x_bounds, y_bounds)


class SurfacePlotFrame(wx.Frame):
	def __init__(self, parent, surface_data, x_bounds, y_bounds, x_label, y_label, z_label,
			style, *args, **kwargs):
		wx.Frame.__init__(self,  parent, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Plot panel.
		self.panel = SurfacePlotPanel(self, surface_data, x_bounds, y_bounds, x_label,
				y_label, z_label, style)
		frame_box.Add(self.panel, proportion=1, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)

		self.Bind(wx.EVT_CLOSE, self.OnClose)

	def OnClose(self, evt):
		self.panel.plot.close()

		evt.Skip()


class SurfacePlotSetupDialog(PlotSetupDialog):
	def __init__(self, parent, headings, data, *args, **kwargs):
		PlotSetupDialog.__init__(self, parent, headings, ['x', 'y', 'z'],
				*args, **kwargs)

		self.parent = parent
		self.headings = headings
		self.data = data

	def make_plot(self):
		try:
			x_data, y_data, z_data = [self.data[:,axis].astype(float) for axis in self.axes]
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()
			return

		try:
			surface_data, x_bounds, y_bounds, _ = triples_to_mesh(x_data, y_data, z_data)
		except Exception as e:
			MessageDialog(self, str(e), 'Conversion failure').Show()
			return

		x_label, y_label, z_label = [self.headings[x] for x in self.axes]
		title = '{0} vs ({1}, {2})'.format(z_label, x_label, y_label)

		frame = SurfacePlotFrame(self.parent, surface_data, x_bounds, y_bounds,
				x_label, y_label, z_label, title=title, style='surface')
		frame.Show()

		return True


class WaveformsPlotSetupDialog(PlotSetupDialog):
	def __init__(self, parent, headings, data, *args, **kwargs):
		PlotSetupDialog.__init__(self, parent, headings, ['z'],
				*args, **kwargs)

		self.parent = parent
		self.headings = headings
		self.data = data

	def make_plot(self):
		axis = self.axes[0]
		lp = ListParser()

		try:
			surface_data = array([[x[1] for x in lp(row)] for row in self.data[:,axis]])
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()
			return

		x_axis = [x[0] for x in lp(self.data[:,axis][0])]
		x_bounds = (x_axis[0], x_axis[-1])

		x_label, y_label, z_label = 'Waveform (s)', 'History', self.headings[axis]
		title = '{0} vs ({1}, {2})'.format(z_label, x_label, y_label)

		frame = SurfacePlotFrame(self.parent, surface_data, x_bounds, (1, len(surface_data)),
				x_label, y_label, z_label, title=title, style='waveform')
		frame.Show()

		return True



import random

from ...mock.mock_abstract_device import MockAbstractDevice
from ..dm34410a import DM34410A

"""
Mock Agilent 34410A Digital Multimeter

Control a fake multimeter.
"""


class MockDM34410A(MockAbstractDevice, DM34410A):
	"""
	Mock interface for Agilent 34410A Digital Multimeter.
	"""

	def __init__(self, *args, **kwargs):
		self.mocking = DM34410A

		MockAbstractDevice.__init__(self, *args, **kwargs)

	def _reset(self):
		self.mock_state['mode'] = 'dc'
		self.mock_state['nplc'] = '1'
		self.mock_state['auto_zero'] = 1

	def write(self, message, result=None, done=False):
		if not done:
			cmd, args, query = self._split_message(message)

			if cmd[0] == 'configure':
				if cmd[1] == 'voltage' and cmd[2] == 'dc':
					self.mock_state['mode'] = 'dc'
					done = True
			elif cmd[0] == 'sense':
				if cmd[1] == 'voltage' and cmd[2] == 'dc':
					if cmd[3] == 'nplc':
						if query:
							result = self.mock_state['nplc']
						else:
							self.mock_state['nplc'] = args
						done = True
					elif cmd[3] == 'zero' and cmd[4] == 'auto':
						if query:
							result = self.mock_state['auto_zero']
						else:
							if args in ['on']:
								value = 1
							elif args in ['once', 'off']:
								value = 0
							self.mock_state['auto_zero'] = value
						done = True
			elif cmd[0] == 'read' and query:
				result = '-1.{0:04d}0000E-02'.format(random.randint(0, 9999))
				done = True

		MockAbstractDevice.write(self, message, result, done)


name = '34410A'
implementation = MockDM34410A



from nose.tools import eq_
from time import time
from unittest import main

from spacq.interface.units import Quantity
from spacq.tests.tool.box import DeviceServerTestCase

from ... import ips120_10


class IPS120_10Test(DeviceServerTestCase):
	def obtain_device(self):
		return DeviceServerTestCase.obtain_device(self, impl=ips120_10.IPS120_10,
				manufacturer='Oxford Instruments', model='IPS120-10')

	def testScenario(self):
		"""
		Change the sweep rate and output field.

		Note: Verification should also be done manually based on the output.
		"""

		ips = self.obtain_device()
		ips.perma_hot = True

		# Turn off the heater for now.
		ips.heater_on = False
		assert not ips.heater_on

		start_field = ips.field

		# Sweep slowly.
		ips.sweep_rate = Quantity(0.1 / 60, 'T.s-1')

		field1 = Quantity(0.005, 'T')

		start_time = time()
		ips.field = field1
		elapsed_time = time() - start_time

		eq_(ips.set_point, field1)
		eq_(ips.field, field1)

		expected_time = 60 * abs(field1 - start_field).value / 0.1
		assert elapsed_time >= expected_time, 'Took {0} s, expected at least {1} s.'.format(elapsed_time, expected_time)

		# Make sure it stayed on.
		assert ips.heater_on

		# Turn it off next time.
		ips.perma_hot = False

		# Sweep quickly.
		ips.sweep_rate = Quantity(0.5 / 60, 'T.s-1')

		field2 = Quantity(-0.015, 'T')

		start_time = time()
		ips.field = field2
		elapsed_time = time() - start_time

		eq_(ips.field, field2)

		expected_time = 60 * abs(field2 - field1).value / 0.5
		assert elapsed_time >= expected_time, 'Took {0} s, expected at least {1} s.'.format(elapsed_time, expected_time)

		# Make sure it turned off.
		assert not ips.heater_on

		ips.perma_hot = True

		# Reset back to zero.
		field3 = Quantity(0.0, 'T')

		start_time = time()
		ips.field = field3
		elapsed_time = time() - start_time

		eq_(ips.field, field3)

		expected_time = 60 * abs(field3 - field2).value / 0.5
		assert elapsed_time >= expected_time, 'Took {0} s, expected at least {1} s.'.format(elapsed_time, expected_time)


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from math import ceil
from numpy import linspace
import struct

from spacq.interface.resources import Resource
from spacq.tool.box import Synchronized

from ..abstract_device import AbstractDevice, AbstractSubdevice
from ..tools import str_to_bool, quantity_wrapped, quantity_unwrapped, BlockData

"""
Tektronix DPO7104 Digital Phosphor Oscilloscope

Control the DPO's settings and input waveforms.
"""


class Channel(AbstractSubdevice):
	"""
	Input channel of the DPO.
	"""

	def _setup(self):
		AbstractSubdevice._setup(self)

		# Resources.
		read_only = ['waveform']
		for name in read_only:
			self.resources[name] = Resource(self, name)

		read_write = ['enabled']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['waveform'].slow = True
		self.resources['waveform'].display_units = 'V'
		self.resources['enabled'].converter = str_to_bool

	def __init__(self, device, channel, *args, **kwargs):
		self.channel = channel

		AbstractSubdevice.__init__(self, device, *args, **kwargs)

	@property
	def acquisition_window(self):
		"""
		The minimum and maximum obtainable values in V.
		"""

		# 10 divisions total.
		max_value = 5 * self.scale.value
		min_value = -max_value

		offset = self.offset.value

		return (min_value + offset, max_value + offset)

	def transform_waveform(self, waveform):
		"""
		Transform some curve data onto the true amplitude interval in V, and intermix time values in s.
		"""

		value_min, value_max = self.device.value_range
		value_diff = value_max - value_min

		real_min, real_max = self.acquisition_window
		real_diff = real_max - real_min

		times = linspace(0, self.device.time_scale.value, len(waveform))

		return [(time, real_diff * float(x - value_min) / value_diff + real_min) for time, x in zip(times, waveform)]

	@property
	def enabled(self):
		"""
		The input state (on/off) of the channel.
		"""

		result = self.device.ask('select:ch{0}?'.format(self.channel))
		return bool(int(result))

	@enabled.setter
	def enabled(self, value):
		self.device.write('select:ch{0} {1}'.format(self.channel, 'on' if value else 'off'))

	@property
	@Synchronized()
	def waveform(self):
		"""
		A waveform acquired by the scope.

		Values are returned in the format [(time1, value1), (time2, value2), ...].
		"""

		self.device.status.append('Getting waveform for channel {0}'.format(self.channel))

		try:
			self.device.data_source = self.channel

			if self.device.fastframe:
				# Only get the last frame.
				frame = self.device.fastframe_count
			else:
				frame = 1

			self.device.fastframe_start = frame
			self.device.fastframe_stop = frame

			# Receive in chunks.
			num_data_points = self.device.record_length
			num_transmissions = int(ceil(num_data_points / self.device.max_receive_samples))

			curve = []
			for i in xrange(num_transmissions):
				self.device.data_start = int(i * self.device.max_receive_samples) + 1
				self.device.data_stop = int((i + 1) * self.device.max_receive_samples)

				curve_raw = self.device.ask_raw('curve?')
				curve.append(BlockData.from_block_data(curve_raw))

			curve = ''.join(curve)

			format_code = self.device.byte_format_letters[self.device.waveform_bytes]
			curve_data = struct.unpack('!{0}{1}'.format(num_data_points, format_code), curve)

			return self.transform_waveform(curve_data)
		finally:
			self.device.status.pop()

	@property
	@quantity_wrapped('V')
	def scale(self):
		"""
		Vertical scale for the channel, as a quantity in V.

		Note: This is for a single division, of which there are 10.
		"""

		return float(self.device.ask('ch{0}:scale?'.format(self.channel)))

	@scale.setter
	@quantity_unwrapped('V')
	def scale(self, value):
		self.device.write('ch{0}:scale {1}'.format(self.channel, value))

	@property
	@quantity_wrapped('V')
	def offset(self):
		"""
		Vertical offset for the channel, as a quantity in V.
		"""

		return float(self.device.ask('ch{0}:offset?'.format(self.channel)))

	@offset.setter
	@quantity_unwrapped('V')
	def offset(self, value):
		self.device.write('ch{0}:offset {1}'.format(self.channel, value))


class DPO7104(AbstractDevice):
	"""
	Interface for Tektronix DPO7104 DPO.
	"""

	byte_format_letters = [None, 'b', 'h']

	# The upper limit to the number of samples to be received per transmission.
	max_receive_samples = 1e7

	allowed_stopafters = ['runstop', 'sequence']
	allowed_waveform_bytes = [1, 2] # Channel data only.
	allowed_fastframe_sums = set(['none', 'average', 'envelope'])

	def _setup(self):
		AbstractDevice._setup(self)

		self.channels = [None] # There is no channel 0.
		for chan in xrange(1, 5):
			channel = Channel(self, chan)
			self.channels.append(channel)
			self.subdevices['channel{0}'.format(chan)] = channel

		# Resources.
		read_write = ['sample_rate', 'time_scale']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['sample_rate'].units = 'Hz'
		self.resources['time_scale'].units = 's'

	@Synchronized()
	def reset(self):
		"""
		Reset the device to its default state.
		"""

		log.info('Resetting "{0}".'.format(self.name))
		self.write('*rst')

	def autoset(self):
		"""
		Autoset the scaling.
		"""

		self.write('autoset execute')

	@property
	def stopafter(self):
		"""
		The acqusition mode.
		"""

		value = self.ask('acquire:stopafter?').lower()

		if value.startswith('runst'):
			return 'runstop'
		elif value.startswith('seq'):
			return 'sequence'

	@stopafter.setter
	def stopafter(self, value):
		if value not in self.allowed_stopafters:
			raise ValueError('Invalid acquisition mode: {0}'.format(value))

		self.write('acquire:stopafter {0}'.format(value))

	@property
	def waveform_bytes(self):
		"""
		Number of bytes per data point in the acquired waveforms.
		"""

		return int(self.ask('wfmoutpre:byt_nr?'))

	@waveform_bytes.setter
	def waveform_bytes(self, value):
		self.write('wfmoutpre:byt_nr {0}'.format(value))

	@property
	def value_range(self):
		"""
		Range of values possible for each data point.
		"""

		# The returned values are signed.
		bits = 8 * self.waveform_bytes - 1
		max_val = 2 ** bits

		return (-max_val, max_val - 1)

	@property
	def acquiring(self):
		"""
		Whether the device is currently acquiring data.
		"""

		result = self.ask('acquire:state?')
		return bool(int(result))

	@acquiring.setter
	def acquiring(self, value):
		self.write('acquire:state {0}'.format(str(int(value))))

	@property
	@quantity_wrapped('Hz')
	def sample_rate(self):
		"""
		The sample rate in s-1.
		"""

		return float(self.ask('horizontal:mode:samplerate?'))

	@sample_rate.setter
	@quantity_unwrapped('Hz')
	def sample_rate(self, value):
		self.write('horizontal:mode:samplerate {0}'.format(value))

	@property
	@quantity_wrapped('s')
	def time_scale(self):
		"""
		The length for a waveform.
		"""

		return float(self.ask('horizontal:divisions?')) * float(self.ask('horizontal:mode:scale?'))

	@time_scale.setter
	@quantity_unwrapped('s')
	def time_scale(self, value):
		self.write('horizontal:mode:scale {0}'.format(value / float(self.ask('horizontal:divisions?'))))

	@property
	def data_source(self):
		"""
		The source from which to transfer data.
		"""

		result = self.ask('data:source?')
		assert len(result) == 3 and result.startswith('CH')

		return int(result[2])

	@data_source.setter
	def data_source(self, value):
		self.write('data:source ch{0}'.format(value))

	@property
	def data_start(self):
		"""
		The first data point to transfer.
		"""

		return int(self.ask('data:start?'))

	@data_start.setter
	def data_start(self, value):
		self.write('data:start {0}'.format(value))

	@property
	def data_stop(self):
		"""
		The last data point to transfer.
		"""

		return int(self.ask('data:stop?'))

	@data_stop.setter
	def data_stop(self, value):
		self.write('data:stop {0}'.format(value))

	@property
	def record_length(self):
		"""
		The number of data points in a waveform.
		"""

		return int(self.ask('horizontal:mode:recordlength?'))

	@Synchronized()
	def acquire(self):
		"""
		Cause the DPO to acquire a single waveform.
		"""

		self.acquiring = True

	@property
	def fastframe(self):
		"""
		Whether fastframe is enabled.
		"""

		return bool(int(self.ask('horizontal:fastframe:state?')))

	@fastframe.setter
	def fastframe(self, value):
		return self.write('horizontal:fastframe:state {0}'.format(int(value)))

	@property
	def fastframe_sum(self):
		"""
		The fastframe summary frame.
		"""

		result = self.ask('horizontal:fastframe:sumframe?').lower()

		if result.startswith('non'):
			return 'none'
		elif result.startswith('env'):
			return 'envelope'
		elif result.startswith('ave'):
			return 'average'
		else:
			ValueError('Unknown summary mode: {0}'.format(result))

	@fastframe_sum.setter
	def fastframe_sum(self, value):
		if value not in self.allowed_fastframe_sums:
			raise ValueError('Invalid summary frame mode: {0}'.format(value))

		return self.write('horizontal:fastframe:state {0}'.format(value))

	@property
	def fastframe_count(self):
		"""
		The number of waveforms to acquire in fastframe mode.
		"""

		return int(self.ask('horizontal:fastframe:count?'))

	@fastframe_count.setter
	def fastframe_count(self, value):
		if value <= 0:
			raise ValueError('Must provide a positive integer, not "{0}"'.format(value))

		self.write('horizontal:fastframe:count {0:d}'.format(value))

	@property
	def fastframe_start(self):
		"""
		The first frame to transfer.
		"""

		return int(self.ask('data:framestart?'))

	@fastframe_start.setter
	def fastframe_start(self, value):
		self.write('data:framestart {0}'.format(value))

	@property
	def fastframe_stop(self):
		"""
		The last frame to transfer.
		"""

		return int(self.ask('data:framestop?'))

	@fastframe_stop.setter
	def fastframe_stop(self, value):
		self.write('data:framestop {0}'.format(value))

	@property
	def acquisitions(self):
		"""
		The number of acquisitions made so far on the oscilloscope.
		"""

		return int(self.ask('acquire:numacq?'))

name = 'DPO7104'
implementation = DPO7104



from nose.tools import assert_raises, eq_
from unittest import main, TestCase

from .. import mock_abstract_device


class MockAbstractDeviceTest(TestCase):
	def testInit(self):
		"""
		Should always succeed.
		"""

		mock_abstract_device.MockAbstractDevice()
		mock_abstract_device.MockAbstractDevice(ip_address='1234')
		mock_abstract_device.MockAbstractDevice(board=12345, pad=67890)

	def testNotImplemented(self):
		"""
		Ensure that an exception is raised for a non-implemented reqest.
		"""

		dev = mock_abstract_device.MockAbstractDevice()

		assert_raises(NotImplementedError, dev.ask_raw, 'This isn\'t right.')

	def testAskRaw(self):
		"""
		Converse briefly with a fake device.
		"""

		dev = mock_abstract_device.MockAbstractDevice()

		msg = dev.ask_raw('*idn?')
		eq_(msg, 'MockAbstractDevice\n')

	def testMulti(self):
		"""
		Ensure that requests for multi-command messages work.
		"""

		dev = mock_abstract_device.MockAbstractDevice()

		expected = ['42', '42', 'MockAbstractDevice']

		dev.multi_command_start()
		dev.ask('system:version?')
		dev.write('*rst')
		dev.write('system:preset')
		dev.ask('system:version?')
		dev.ask('*idn?')
		result = dev.multi_command_stop()

		eq_(result, expected)


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from os import path

from spacq.tool.box import Enum

from ..units import IncompatibleDimensions, Quantity
from ..waveform import Generator
from .tool.box import find_location, format_error, load_values

"""
Abstract syntax tree bits for pulse programs.
"""


def draw_thing(thing, depth):
	"""
	Draws something at a given depth.
	"""

	if isinstance(thing, ASTNode):
		if thing.is_list:
			result = ('{0}\n'.format(thing.__class__.__name__) +
				''.join(draw_thing(item, depth + 1) for item in thing.items))
		else:
			result = thing.draw(depth)
	else:
		result = str(thing) + '\n'

	return ' ' * depth + result


class Environment(object):
	"""
	An AST-traversal environment.
	"""

	# Traversal stages:
	#  declarations: collect variable names and types
	#  values: collect variable values
	#  commands: verify commands for outputs
	#  waveforms: generate waveforms
	stages = Enum(['declarations', 'values', 'commands', 'waveforms'])

	# The stages that must happen (in this order) before waveform generation.
	# These only require the data which is contained in the pulse program itself.
	prep_stages = [stages.declarations, stages.values, stages.commands]

	def __init__(self):
		self.stage = None
		self.stack = []

		# Declared variable names (with types as values).
		self.variables = {}

		# Variable and attribute values.
		#  Keys are tuples of strings:
		#   ('x',): x is a variable
		#   ('x', 'y'): y is an attribute
		self.values = {}

		# Whether acquisition has already been requested.
		self.acquisition = False

		# Any errors; each error is of the form
		#  (error string, (row, col, code line))
		self.errors = []

		# Whether to actually generate waveforms.
		self.dry_run = False

		# Waveform generators for the output channels.
		# Keys are output names.
		self.generators = {}

		# Generated waveforms.
		self.waveforms = {}

		# Where to look for shapes.
		self.cwd = None

		# Shapes that could not be found.
		self.missing_shapes = set()

		# Default frequency.
		self.frequency = Quantity(1, 'Hz')

	@property
	def missing_values(self):
		existing_values = set(self.values.keys())

		return self.all_values - existing_values

	def add_error(self, msg, loc=None):
		"""
		Add an error.
		"""

		self.errors.append((msg, loc))

	def pre_stage(self):
		"""
		Set up for a stage.
		"""

		if self.stage == self.stages.waveforms:
			if self.missing_values:
				values = ', '.join('.'.join(x) for x in sorted(self.missing_values))

				raise ValueError('Cannot generate waveforms while values are missing: {0}'.format(values))

			# Set up output waveform generators.
			for output in self.waveforms:
				self.generators[output] = Generator(frequency=self.frequency, dry_run=self.dry_run)

	def post_stage(self):
		"""
		Clean up after a stage has completed.
		"""

		if self.stage == self.stages.declarations:
			# Prepare for output waveform generators.
			for output in [var for var, type in self.variables.items() if type == 'output']:
				self.generators[output] = None
				self.waveforms[output] = None

			# Generate labels for all necessary values.
			self.all_values = set()
			for name, type in self.variables.items():
				if type == 'pulse':
					for attr in ['amplitude', 'length', 'shape']:
						self.all_values.add((name, attr))
				elif type == 'acq_marker':
					for attr in ['marker_num', 'output']:
						self.all_values.add((name, attr))
				elif type != 'output':
					self.all_values.add((name,))
		elif self.stage == self.stages.waveforms:
			# Finalize waveform creation.
			for output in self.generators:
				self.waveforms[output] = self.generators[output].waveform

	def set_value(self, target, value):
		"""
		Set a value if the types work out. TypeError otherwise.
		"""

		# Type checking.
		var_type = self.variables[target[0]]
		if var_type == 'acq_marker':
			if len(target) == 1:
				raise TypeError('Must assign dictionary to acq_marker')
			else:
				if target[1] == 'marker_num':
					if not isinstance(value, int):
						raise TypeError('Must assign int to acq_marker num')
				elif target[1] == 'output':
					if not isinstance(value, basestring):
						raise TypeError('Must assign string to acq_marker num')
		elif var_type == 'delay':
			try:
				value.assert_dimensions('s')
			except (AttributeError, IncompatibleDimensions):
				raise TypeError('Must assign time quantity to delay')
		elif var_type == 'int':
			if not isinstance(value, int):
				raise TypeError('Must assign integer to int')
		elif var_type == 'pulse':
			if len(target) == 1:
				raise TypeError('Must assign dictionary to pulse')
			else:
				if target[1] == 'amplitude':
					try:
						value.assert_dimensions('V')
					except (AttributeError, IncompatibleDimensions):
						raise TypeError('Must assign voltage quantity to pulse amplitude')
				elif target[1] == 'length':
					try:
						value.assert_dimensions('s')
					except (AttributeError, IncompatibleDimensions):
						raise TypeError('Must assign time quantity to pulse length')
				elif target[1] == 'shape':
					if not isinstance(value, basestring):
						raise TypeError('Must assign string to pulse shape')
		else:
			raise TypeError('Cannot assign to variable of type "{0}"'.format(var_type))

		if target in self.values and self.values[target] is not None:
			raise TypeError('Re-assignment of {0}'.format(target))
		else:
			self.values[target] = value

	def traverse_tree(self, root):
		"""
		Modify the Environment, given a pulse program AST.
		"""

		self.pre_stage()
		root.visit(self)
		self.post_stage()

	def format_errors(self):
		result = []

		# Sort by line number, then column number, and ignore duplicates.
		for error in sorted(set(self.errors),
				key=(lambda x: (x[1][0], x[1][1]) if x[1] is not None else (-1, -1))):
			if error[1] is not None:
				result.append(format_error(error[0], *error[1]))
			else:
				result.append(format_error(error[0]))

		return result


class ASTNode(object):
	names = []
	is_list = False

	def __init__(self, *args):
		log.debug('Creating node of type "{0}".'.format(self.__class__.__name__))

		if len(args) == 3:
			self.s = args[0]
			self.loc = args[1]

			tok = args[2]
		else:
			self.s = None
			self.loc = None

			if len(args) == 1:
				tok = args[0]
			else:
				tok = None

		log.debug('Received tokens: {0!r}'.format(tok))

		if self.is_list:
			self.items = list(tok)
		else:
			for name in self.names:
				setattr(self, name, tok[name])

	def __eq__(self, other):
		return repr(self) == repr(other)

	def draw(self, depth):
		return ' ' * depth + self.__class__.__name__ + '\n'

	@property
	def location(self):
		return find_location(self.s, self.loc)


class Acquire(ASTNode):
	def __repr__(self):
		return 'acquire'

	def visit(self, env):
		if env.stage == env.stages.declarations:
			if len(env.stack) != 1:
				env.add_error('Acquisition not at top level', self.location)

			env.variables['_acq_marker'] = 'acq_marker'
		if env.stage == env.stages.commands:
			if env.acquisition:
				env.add_error('Repeated acquisition', self.location)
			else:
				env.acquisition = True
		elif env.stage == env.stages.waveforms:
			acq_marker = env.values[('_acq_marker', 'marker_num')]
			acq_output = env.values[('_acq_marker', 'output')]

			env.generators[acq_output].marker(acq_marker, True)


class Assignment(ASTNode):
	names = ['target', 'value']

	def __repr__(self):
		return '{0!r} = {1!r}'.format(self.target, self.value)

	def draw(self, depth):
		return 'Assignment\n' + draw_thing(self.target, depth + 1) + draw_thing(self.value, depth + 1)

	def assign_value(self, env, target, value):
		try:
			env.set_value(target, value)
		except KeyError as e:
			env.add_error('Undeclared variable "{0}"'.format(e), self.location)
		except TypeError as e:
			env.add_error(str(e), self.location)

	def visit(self, env):
		if env.stage == env.stages.declarations:
			if isinstance(self.target, ASTNode):
				return self.target.visit(env)
			else:
				return (self.target,)
		elif env.stage == env.stages.values:
			if len(env.stack) != 1 and env.stack[-1].__class__ != Declaration:
				env.add_error('Assignment neither at top level nor inside declaration', self.location)

			if isinstance(self.target, ASTNode):
				env.stack.append(self)
				target = self.target.visit(env)
				env.stack.pop()
			else:
				target = (self.target,)

			if target[0] not in env.variables:
				env.add_error('Undeclared variable "{0}"'.format(target[0]), self.location)
			else:
				if isinstance(self.value, Dictionary):
					for k, v in self.value.visit(env).items():
						self.assign_value(env, target + (k,), v)
				else:
					self.assign_value(env, target, self.value)


class Attribute(ASTNode):
	names = ['variable', 'name']

	def __repr__(self):
		return '{0!r}.{1!r}'.format(self.variable, self.name)

	def draw(self, depth):
		return 'Attribute\n' + draw_thing(self.variable, depth + 1) + draw_thing(self.name, depth + 1)

	def visit(self, env):
		result = (self.variable, self.name)

		if env.stage == env.stages.values:
			if result not in env.all_values:
				env.add_error('Unrecognized attribute "{0}"'.format('.'.join(result)), self.location)

		return result


class Block(ASTNode):
	is_list = True

	def __repr__(self):
		return '{{{0}}}'.format(', '.join(repr(item) for item in self.items))

	def draw(self, depth=0):
		# A default of 0 allows this to be called on the AST as a whole.

		return draw_thing(self, depth)

	def visit(self, env):
		env.stack.append(self)
		for item in self.items:
			item.visit(env)
		env.stack.pop()


class Declaration(ASTNode):
	names = ['type', 'variables']

	def __repr__(self):
		return '{0!r} {1}'.format(self.type, ', '.join(repr(variable) for variable in self.variables))

	def draw(self, depth):
		result = ['Declaration\n' + draw_thing(self.type, depth + 1)]

		for variable in self.variables:
			result.append(draw_thing(variable, depth + 1))

		return ''.join(result)

	def visit(self, env):
		if env.stage == env.stages.declarations:
			if len(env.stack) != 1:
				env.add_error('Declaration not at top level', self.location)

			for variable in self.variables:
				env.stack.append(self)
				visited = variable.visit(env)
				env.stack.pop()

				if env.stage == env.stages.declarations:
					new_var_name = visited[0]

					if new_var_name in env.variables:
						env.add_error('Re-declaration of "{0}"'.format(new_var_name), variable.location)
					else:
						env.variables[new_var_name] = self.type
		else:
			for variable in self.variables:
				env.stack.append(self)
				visited = variable.visit(env)
				env.stack.pop()


class Delay(ASTNode):
	names = ['length']

	def __repr__(self):
		return '{0!r}'.format(self.length)

	def draw(self, depth):
		return 'Delay\n' + draw_thing(self.length, depth + 1)

	def visit(self, env):
		if env.stage == env.stages.commands:
			if isinstance(self.length, basestring):
				try:
					if env.variables[self.length] != 'delay':
						env.add_error('Not a delay', self.location)
				except KeyError:
					env.add_error('Undeclared variable "{0}"'.format(self.length), self.location)
			else:
				if not self.length.assert_dimensions('s', exception=False):
					env.add_error('Delay must be a time value', self.location)
		if env.stage == env.stages.waveforms:
			if isinstance(self.length, basestring):
				length = env.values[(self.length,)]
			else:
				length = self.length

			for waveform in env.generators.values():
				waveform.set_next(0.0)
				waveform.delay(length)


class Dictionary(ASTNode):
	is_list = True

	def __repr__(self):
		return '{{{0}}}'.format(', '.join(repr(item) for item in self.items))

	def visit(self, env):
		return dict(x.visit(env) for x in self.items)


class DictionaryItem(ASTNode):
	names = ['key', 'value']

	def __repr__(self):
		return '{0!r}: {1!r}'.format(self.key, self.value)

	def draw(self, depth):
		return repr(self) + '\n'

	def visit(self, env):
		return (self.key, self.value)


class Loop(ASTNode):
	names = ['times', 'block']

	def __repr__(self):
		return 'times {0!r} {1!r}'.format(self.times, self.block)

	def draw(self, depth):
		return 'Loop\n' + draw_thing(self.times, depth + 1) + draw_thing(self.block, depth + 1)

	def visit(self, env):
		if env.stage == env.stages.commands:
			if isinstance(self.times, basestring):
				try:
					if env.variables[self.times] != 'int':
						env.add_error('Repetition count must be int', self.location)
				except KeyError:
					env.add_error('Undeclared variable "{0}"'.format(self.times), self.location)

		if env.stage == env.stages.waveforms:
			if isinstance(self.times, basestring):
				times = env.values[(self.times,)]
			else:
				times = self.times

			for _ in xrange(times):
				env.stack.append(self)
				self.block.visit(env)
				env.stack.pop()
		else:
			env.stack.append(self)
			self.block.visit(env)
			env.stack.pop()


class ParallelPulses(ASTNode):
	is_list = True

	def __repr__(self):
		return '{0}'.format(' '.join(repr(item) for item in self.items))

	def visit(self, env):
		env.stack.append(self)
		for item in self.items:
			item.visit(env)
		env.stack.pop()

		if env.stage == env.stages.waveforms:
			max_length = max(len(waveform._wave) for waveform in env.generators.values())

			for waveform in env.generators.values():
				if len(waveform._wave) < max_length:
					waveform.append([0.0] * (max_length - len(waveform._wave)))


class Pulse(ASTNode):
	names = ['sequence', 'target']

	def __repr__(self):
		return '{0!r}:{1!r}'.format(self.sequence, self.target)

	def draw(self, depth):
		return 'Pulse\n' + draw_thing(self.sequence, depth + 1) + draw_thing(self.target, depth + 1)

	def visit(self, env):
		if env.stage == env.stages.commands:
			if self.target not in env.generators:
				env.add_error('Undefined output "{0}"'.format(self.target), self.location)

		env.stack.append(self)
		self.sequence.visit(env)
		env.stack.pop()


class PulseSequence(ASTNode):
	is_list = True

	def __repr__(self):
		return '({0})'.format(', '.join(repr(item) for item in self.items))

	def visit(self, env):
		if env.stage == env.stages.commands:
			for item in self.items:
				if isinstance(item, Delay):
					if not item.length.assert_dimensions('s', exception=False):
						env.add_error('Delay must be a time value', self.location)

					continue

				try:
					type = env.variables[item]
				except KeyError:
					env.add_error('Undeclared variable "{0}"'.format(item), self.location)
				else:
					if type not in ['delay', 'pulse']:
						env.add_error('Invalid command "{0}"'.format(item), self.location)
		elif env.stage == env.stages.waveforms:
			target = env.generators[env.stack[-1].target]

			for item in self.items:
				if isinstance(item, basestring):
					type = env.variables[item]

					if type == 'delay':
						target.set_next(0.0)
						target.delay(env.values[(item,)])
					elif type == 'pulse':
						amplitude = float(env.values[(item, 'amplitude')].value)
						length = env.values[(item, 'length')]
						shape = env.values[(item, 'shape')]

						if shape not in env.missing_shapes:
							if shape == 'square':
								target.square(amplitude, length)
							else:
								# Figure out all the locations where the file can be.
								paths = [shape]
								if env.cwd is not None:
									paths.append(path.join(env.cwd, shape))

								data = None
								for p in paths:
									try:
										with open(p) as f:
											data = load_values(f)
									except IOError:
										continue
									except ValueError:
										raise ValueError('Not a shape file: {0}'.format(p))

								if data is None:
									env.add_error('File "{0}" (due to "{1}") not found'.format(shape, item),
											self.location)
									env.missing_shapes.add(shape)
								else:
									target.pulse(data, amplitude, length)
				else:
					target.set_next(0.0)
					target.delay(item.length)


class Variable(ASTNode):
	names = ['name']

	def __repr__(self):
		return '{0!r}'.format(self.name)

	def draw(self, depth):
		return 'Variable\n' + draw_thing(self.name, depth + 1)

	def visit(self, env):
		return (self.name,)



import csv

from spacq.tool.box import flatten

"""
Pulse program helpers.
"""


def find_location(s, loc):
	"""
	Find where loc (linear index) in s.

	Returns:
		line number
		column number
		line itself

	Note: Tabs are not handled specially.
	"""

	if loc < 0 or loc > len(s):
		raise ValueError('Location given ({0}) is outside string'.format(loc))

	count = 0
	lines = s.splitlines()
	with_ends = s.splitlines(True)

	for i, (line, with_end) in enumerate(zip(lines, with_ends), 1):
		if count + len(line) < loc:
			count += len(with_end)
		else:
			col = loc - count + 1

			return i, col, line


def format_error(msg, row=None, col=None, line=None):
	"""
	Format the error for human consumption.
	"""

	if row is None or col is None or line is None:
		return 'error: {0}'.format(msg)
	else:
		return 'error: {0} at column {1} on line {2}:\n{3}{4}\n{5}^'.format(msg,
				col, row, ' ' * 2, line, ' ' * (col + 1))


def load_values(f):
	"""
	Load data points from a file.

	The values in the file must either be comma separated, line-wise, or a combination of the two.
	For example:
		1.0,2.0,3.0
		4.0,5.0
		6.0
	would be interpreted as [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
	"""

	reader = csv.reader(f)

	# Ignore blank lines.
	return [float(x) for x in flatten(reader) if not x.isspace()]



from functools import partial
from threading import Thread
import wx
from wx.gizmos import TreeListCtrl

from spacq.interface.resources import NotReadable
from spacq.interface.units import IncompatibleDimensions

from ...tool.box import MessageDialog

"""
A tree of subdevices and resources.
"""


class ResourceFetcher(Thread):
	"""
	A thread which iterates over a list of items, getting resource values.
	"""

	def __init__(self, items, getter, callback, *args, **kwargs):
		"""
		items: List of TreeListCtrl items.
		getter: Given an item, returns a Resource or None.
		callback: Is called with the item and the value of the resource.
		"""

		Thread.__init__(self, *args, **kwargs)

		self.items = items
		self.getter = getter
		self.callback = callback

	def run(self):
		try:
			for item in self.items:
				resource = self.getter(item)

				if resource is not None and resource.readable:
					if resource.slow:
						wx.CallAfter(self.callback, item, '[N/A]')
					else:
						wx.CallAfter(self.callback, item, resource.value)
		except wx.PyDeadObjectError:
			# The values are no longer wanted.
			return


class ItemData(object):
	"""
	Useful information about a node to be stored in its PyData.
	"""

	def __init__(self, path, resource):
		self.path = path
		self.resource = resource

		self.fetched = False


class ResourceTree(TreeListCtrl):
	"""
	A tree list to display an hierarchy of subdevices and resources.
	"""

	def __init__(self, parent, *args, **kwargs):
		TreeListCtrl.__init__(self, parent, *args,
				style=wx.TR_DEFAULT_STYLE|wx.TR_FULL_ROW_HIGHLIGHT|wx.TR_HIDE_ROOT,
				**kwargs)

		self.root = None
		self.resource_labels = []

		self.col_name = 0
		self.AddColumn('Name', 200)
		self.col_r = 1
		self.AddColumn('R', 24)
		self.col_w = 2
		self.AddColumn('W', 24)
		self.col_units = 3
		self.AddColumn('Units', 50)
		self.col_label = 4
		self.AddColumn('Label', 200, edit=True)
		self.col_value = 5
		self.AddColumn('Value', 400, edit=True)

		# Extra 50 for nesting.
		self.SetMinSize((950, -1))

		self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.OnItemExpanded)
		self.Bind(wx.EVT_TREE_BEGIN_LABEL_EDIT, self.OnBeginLabelEdit)
		self.Bind(wx.EVT_TREE_END_LABEL_EDIT, self.OnEndLabelEdit)
		self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnActivated)

	def GetChildren(self, item):
		"""
		Non-recursive generator for the children of an item.
		"""

		if self.HasChildren(item):
			child, cookie = self.GetFirstChild(item)

			while child:
				yield child

				child, cookie = self.GetNextChild(item, cookie)

	def GetLeaves(self, item=None):
		"""
		Recursively collect the leaves under an item.
		"""

		if item is None:
			if self.root is None:
				return []
			else:
				item = self.root

		if not self.HasChildren(item):
			return [item]
		else:
			result = []

			for child in self.GetChildren(item):
				result.extend(self.GetLeaves(child))

			return result

	def fell(self):
		"""
		Cut down the tree.
		"""

		self.DeleteAllItems()
		self.root = None

	def spawn_fetch_thread(self, items):
		"""
		Create a thread to populate the items.
		"""

		def fetch(item):
			pydata = self.GetItemPyData(item)

			if pydata is not None:
				if not pydata.fetched:
					pydata.fetched = True

					return pydata.resource

		def set(item, value):
			try:
				self.SetItemText(item, str(value), self.col_value)
			except wx.PyDeadObjectError:
				# The value isn't wanted anymore.
				return

		thr = ResourceFetcher(items, fetch, set)
		thr.daemon = True
		thr.start()

	def build_tree(self, device, resource_labels, root=None, path=None):
		"""
		Recursively append all subdevices and resources.
		"""

		if root is None:
			self.fell()

			self.root = self.AddRoot('')
			root = self.root

			path = ()

		for name, subdev in device.subdevices.items():
			item = self.AppendItem(root, name)
			full_path = path + (name,)

			self.build_tree(subdev, resource_labels, item, full_path)

		for name, resource in device.resources.items():
			item = self.AppendItem(root, name)
			full_path = path + (name,)

			if resource.getter is not None:
				self.SetItemText(item, 'R', self.col_r)

			if resource.setter is not None:
				self.SetItemText(item, 'W', self.col_w)

			if resource.display_units is not None:
				self.SetItemText(item, resource.display_units, self.col_units)

			self.SetItemPyData(item, ItemData(full_path, resource))

			if full_path in resource_labels:
				self.SetItemText(item, resource_labels[full_path], self.col_label)

		self.SortChildren(root)

		if root == self.root:
			self.spawn_fetch_thread(self.GetChildren(self.root))

	def set_value(self, item, value, error_callback=None):
		"""
		Set the value of a resource, as well as the displayed value.
		"""

		pydata = self.GetItemPyData(item)
		resource = pydata.resource

		def update():
			try:
				resource.value = resource.convert(value)
			except IncompatibleDimensions:
				if error_callback is not None:
					error_callback(ValueError('Expected dimensions to match "{0}"'.format(resource.units)))
				else:
					raise
			except Exception as e:
				if error_callback is not None:
					error_callback(e)
				else:
					raise

			try:
				true_value = str(resource.value)
			except NotReadable:
				pass
			else:
				wx.CallAfter(self.SetItemText, item, true_value, self.col_value)

		thr = Thread(target=update)
		thr.daemon = True
		thr.start()

	def OnItemExpanded(self, evt):
		"""
		Get any resources which may now be visible.
		"""

		self.spawn_fetch_thread(self.GetChildren(evt.Item))

	def OnBeginLabelEdit(self, evt):
		# EVT_TREE_END_LABEL_EDIT does not carry this value.
		self.editing_col = evt.Int

		if evt.Int == self.col_label:
			# Only resources can have labels.
			if not (self.GetItemText(evt.Item, self.col_r) or
					self.GetItemText(evt.Item, self.col_w)):
				evt.Veto()
			else:
				self.old_label = self.GetItemText(evt.Item, self.col_label)
		elif evt.Int == self.col_value:
			# Can only write to writable resources.
			if not self.GetItemText(evt.Item, self.col_w):
				evt.Veto()

			pydata = self.GetItemPyData(evt.Item)
			resource = pydata.resource

			if resource.allowed_values is not None:
				options = [str(x) for x in sorted(resource.allowed_values)]

				dlg = wx.SingleChoiceDialog(self, '', 'Choose value', options)
				# Select the current value if possible.
				try:
					dlg.SetSelection(options.index(self.GetItemText(evt.Item, self.col_value)))
				except ValueError:
					pass

				if dlg.ShowModal() == wx.ID_OK:
					try:
						self.set_value(evt.Item, dlg.GetStringSelection())
					except ValueError as e:
						MessageDialog(self, str(e), 'Invalid value').Show()
						return

				# No need for the editor.
				evt.Veto()
		else:
			evt.Veto()

	def OnEndLabelEdit(self, evt):
		if self.editing_col == self.col_label:
			# Prevent duplicates.
			value = evt.Label

			# Don't do anything if unchanged.
			if value != self.old_label:
				if value not in self.resource_labels:
					if self.old_label:
						self.resource_labels.remove(self.old_label)
					if value:
						self.resource_labels.append(value)
				else:
					evt.Veto()
					MessageDialog(self, str(value), 'Duplicate label').Show()
					return
		elif self.editing_col == self.col_value:
			# Update the real value.
			value = evt.Label

			def error_callback(e):
				MessageDialog(self, str(e), 'Invalid value').Show()

			self.set_value(evt.Item, value, error_callback=partial(wx.CallAfter, error_callback))

	def OnActivated(self, evt):
		"""
		Double click to edit.
		"""

		self.EditLabel(evt.Item, evt.Int)


class DeviceResourcesPanel(wx.Panel):
	"""
	A panel for displaying the subdevices and resources of a device.
	"""

	def __init__(self, parent, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Tree.
		self.tree = ResourceTree(self)
		panel_box.Add(self.tree, proportion=1, flag=wx.EXPAND)

		self.SetSizer(panel_box)

	def set_device(self, device, resource_labels):
		if device is None:
			self.tree.fell()
		else:
			self.tree.build_tree(device, resource_labels)

	def GetValue(self):
		labels = {}
		resources = {}

		for leaf in self.tree.GetLeaves():
			pydata = self.tree.GetItemPyData(leaf)
			name = self.tree.GetItemText(leaf, self.tree.col_label)

			if name:
				labels[pydata.path] = name
				resources[name] = pydata.resource

		return (labels, resources)

	def SetValue(self, resource_labels, resources):
		for path, name in resource_labels.items():
			for leaf in self.tree.GetLeaves():
				pydata = self.tree.GetItemPyData(leaf)

				if pydata.path == path:
					self.tree.SetItemText(leaf, name, self.tree.col_label)

		self.tree.resource_labels = resource_labels.values()



import logging
log = logging.getLogger(__name__)

from spacq.interface.resources import Resource
from spacq.tool.box import Synchronized

from ..abstract_device import AbstractDevice
from ..tools import quantity_wrapped

"""
Sample ABC1234 Device
"""


class ABC1234(AbstractDevice):
	"""
	Interface for the Sample ABC1234.
	"""

	allowed_settings = ['default value', 'something else', '...']

	def _setup(self):
		AbstractDevice._setup(self)

		# Resources.
		read_only = ['reading']
		for name in read_only:
			self.resources[name] = Resource(self, name)

		read_write = ['setting']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['reading'].units = 'A'
		self.resources['setting'].allowed_values = self.allowed_settings

	@Synchronized()
	def _connected(self):
		AbstractDevice._connected(self)

		# Override the default.
		self.setting = '...'

	@Synchronized()
	def reset(self):
		log.info('Resetting "{0}".'.format(self.name))
		self.write('*rst')

	@property
	def setting(self):
		"""
		This is a generic setting.
		"""

		return self.ask('some:setting?')

	@setting.setter
	def setting(self, value):
		if value not in self.allowed_settings:
			raise ValueError('Invalid setting: {0}'.format(value))

		self.write('some:setting {0}'.format(value))

	@property
	@quantity_wrapped('A')
	@Synchronized()
	def reading(self):
		"""
		The value measured by the device.
		"""

		log.debug('Getting reading.')
		result = float(self.ask('read?'))
		log.debug('Got reading: {0}'.format(result))

		return result


name = 'ABC1234'
implementation = ABC1234



import logging
log = logging.getLogger(__name__)


name = 'Agilent'

from . import dm34410a
models = [dm34410a]
log.debug('Found models for "{0}": {1}'.format(name, ''.join(str(x) for x in models)))

from .mock import mock_dm34410a
mock_models = [mock_dm34410a]
log.debug('Found mock models for "{0}": {1}'.format(name, ''.join(str(x) for x in mock_models)))
